/* Read the data-files and parse it */
#include "xrisk.h"
#include "risklibex.h"

/* Loads a run-lengt-encoded file. Returns pointer to the result ( or NULL) */
char *load(filename, x, y)
     char *filename;
     int *x, *y;
{
  int   count,tot;
  byte  ant;
  char *buffer,code;
  FILE *from;

  if (!(from=fopen(filename,"r"))) return(NULL);
  Fgetint(*x,from,count);
  Fgetint(*y,from,count);
  buffer = (char *)malloc((*x)*(*y));
  count  = 0;
  tot    = (*x)*(*y);
  do
    {
      fread((char *)&ant,sizeof(ant),1,from);
      fread((char *)&code,sizeof(code),1,from);
      for(;(ant);ant--)	buffer[count++]=code;
      if(count<tot)
	{
	  fread((char *)&ant,sizeof(ant),1,from);
	  fread((char *)&buffer[count],sizeof(*buffer),(int)ant,from);
	  count+=(int)ant;
	}
    }
  while(tot>count);
  if (tot!=count) fprintf(stderr,"Error in run-length-coding : %s\n",filename);
  fclose(from);
  return(buffer);
}

/* Get the specified picture. Returns pointer to the result an size */
char *get_picture(filename, fx, fy)
     char *filename;
     int fx, fy;
{
  int          x,y;
  char         temp[10000],*picture;
  static char *xrasdir,flag=0;

  if (!flag)
    {
      if (! (xrasdir=getenv("XRISK_XRASDIR")) ) 
	xrasdir=strcpy(malloc(strlen(XRASDIR)+1),XRASDIR);
      flag=1;
    }
  
  if (xrasdir[strlen(xrasdir)-1] == '/')
    sprintf(temp,"%s%s%s",xrasdir,filename,EXTENTION);
  else
    sprintf(temp,"%s/%s%s",xrasdir,filename,EXTENTION);
  
  if (!(picture=load(temp,&x,&y))) 
    risk_exit("Could not find the picture : %s\n",temp,LAST_ARG);
  if ((x!=fx)||(y!=fy)) risk_exit
    ("Wrong size of picture : %s  Is: %d x %d Should be : %d x $d \n"
     ,temp,x,y,fx,fy,LAST_ARG);
  return(picture);
}

/* Returns pointer to continent with name s */
continentp find_continent_string(s)
char *s;
{
  int t;
  
  for(t=0;((t<NUM_CONTINENTS)&&(strcmp(s,continents[t].name)));t++);
  if(t!=NUM_CONTINENTS) return(&continents[t]);
  return(NULL);
}

/* Returns pointer to continent with name s */
countryp find_country_string(s)
     char *s;
{
  int t;
  
  for(t=0;((t<NUM_COUNTRIES)&&(strcmp(s,countries[t].name)));t++);
  if(t!=NUM_COUNTRIES) return(&countries[t]);
  return(NULL);
}

/* Returns number of char c in the streng s */
int num_char(s,c)
     char *s,c;
{
  int t;
  for(t=0;*s;s++) if (*s==c) t++;
  return(t);
}

/* Read a line from the datafile, comments (starting with "!) are ignored */
void read_line(buffer,fd)
     char *buffer;
     FILE *fd;
{
  *buffer = 0;
  do if (!fgets(buffer,MAXLEN_MISC,fd))
    risk_exit("Unexpected eof in datfile (But which ?)\n",LAST_ARG);
  while((*buffer==0)||(*buffer=='!'));
  *strchr(buffer,'\n')=0;
}

/* Read and convert information from datafile              */
/* Some of the information must be treated specially later */
void read_files()
{
  FILE *fil;
  int   count,t,t1,ant[NUM_CONTINENTS];
  char  buffer[MAXLEN_MISC+1],*s;
  char *datafil,mapfile[MAXLEN_MISC+1],*mapdir,*s1,*s2;

  if (! (datafil=getenv("XRISK_DATAFILE")) ) 
    datafil=strcpy(malloc(strlen(DATAFILE)+1),DATAFILE);
  
  if (!(fil=fopen(datafil,"r"))) 
    risk_exit("Missing file %s\n",datafil,LAST_ARG);
  
  /* Read colors */
  for(t=0;t<MAX_PLAYERS;t++)
    {
      read_line(buffer,fil);
      colors[t].flags = DoRed|DoGreen|DoBlue;
      sscanf(buffer,"%hu%hu%hu",
	     &(colors[t].red),&(colors[t].green),&(colors[t].blue));
      if ((colors[t].red+colors[t].green+colors[t].blue <= 0 )
	  ||(colors[t].red+colors[t].green+colors[t].blue > 255*3 ))
	risk_exit("Mistake in datafile (Color) %d (%s)\n",t,buffer,LAST_ARG);
      colors[t].red  <<=8;
      colors[t].green<<=8;
      colors[t].blue <<=8;
    }
  
  /* Information about continents */
  for(t=0;t<NUM_CONTINENTS;t++)
    {
      read_line(buffer,fil);
      sscanf(buffer,"%s%d%d",continents[t].name,&continents[t].weight,&ant[t]);
      if (!ant[t]) risk_exit("Mistake in datafil (Numb. of countries) %s\n",
			     datafil,LAST_ARG);
    }
  
  /* More information about continents */
  for(count=0,t=0;t<NUM_CONTINENTS;t++)
    {
      continents[t].countries=
	(struct Country **)malloc((ant[t]+1)*sizeof(countryp));
      for(t1=0;t1<ant[t];t1++)
	{
	  read_line(buffer,fil);
	  countries[count].neighbours=(struct Country **)malloc(MAXLEN_MISC+1);
	  sscanf(buffer,"%s%d%d%s%s",
		 countries[count].name,
		 &countries[count].x,&countries[count].y,
		 countries[count].unit,
		 (char *)countries[count].neighbours);
	  countries[count].indexl     = count;
	  countries[count].continent  = &continents[t];
	  continents[t].countries[t1] = &(countries[count++]);
	}
      continents[t].countries[t1] = NULL;
    }
  if (count!=NUM_COUNTRIES) risk_exit
    ("Mistake in datafile (Numb. of countries II) %d\n",count,LAST_ARG);
  
  /* Read the missions */
  read_line(buffer,fil);
  if (!(s1=strchr(buffer,':'))) 
    risk_exit("Mistake in datafile (Mission):%s\n",buffer,LAST_ARG);
  
  *(s1++)=0;
  sscanf(s1,"%d",&t1);
  if (strcmp(buffer,"MISSIONS")||(t1<0)||(t1+MAX_PLAYERS> MAX_MISSIONS)) 
    risk_exit("Mistake in datafile(mission):%s [%d]\n",buffer,t1,LAST_ARG);
  
  for(t=0;t<t1;t++)
    {
      read_line(buffer,fil);
      missions[t]                 = (missionp)malloc(sizeof(struct Mission));
      missions[t]->victim         = 0;
      missions[t]->sentence[0]    = 0;
      missions[t]->conquestscont  = 0;
      missions[t]->conquestsc     = 0;
      missions[t]->army           = 0;
      
      if (!(s1=strchr(buffer,':'))) 
	risk_exit("Mistake in datafile (Mission):%s\n",buffer,LAST_ARG);
      
      *(s1++)=0;
      if(!strcmp(buffer,"Country"))
	sscanf(s1,"%d%d",&missions[t]->conquestsc,&missions[t]->army);
      else if(!strcmp(buffer,"Continent"))
	{
	  strcpy((s2=(char *)malloc(MAXLEN_MISC+1)),s1);
	  missions[t]->conquestscont=(struct Continent **)s2;
	}
      else risk_exit("Mistake in datafile(mission):%s\n",buffer,LAST_ARG);
    }
  missions[t] = (missionp)NULL;
  
  /* Read the language-support-file */
  if (! (mapdir=getenv("XRISK_MAPDIR")) ) 
    mapdir=strcpy(malloc(strlen(MAPDIR)+1),MAPDIR);

  if (mapdir[strlen(mapdir)-1] == '/')
    sprintf(mapfile,"%s%s.mapping",mapdir,game_options.language);
  else
    sprintf(mapfile,"%s/%s.mapping",mapdir,game_options.language);
  if (!(fil=fopen(mapfile,"r"))) 
    risk_exit("Missing file %s\n",mapfile,LAST_ARG);
  for(t=0;t<MAX_MAPS;t++) sentencemapping[t]=0;
  do
    {
      read_line(buffer,fil);
      t = mapnumber(&buffer[1]);
      if (t>=0) 
	{
	  if (sentencemapping[t]) risk_exit
	    ("Mapnummer %d exists twice : %s\n",t,mapfile,LAST_ARG);
	  if ( (s=strchr(buffer,'@') + 2) == (char *)2) risk_exit
	    ("Missing @ in sentence %s ,file %s\n",buffer,mapfile,LAST_ARG);
	  if (s[strlen(s)-1] == '"') s[strlen(s)-1] = 0;
	  else risk_exit
	    ("Mistake in sentence %s, in file %s\n",buffer,mapfile,LAST_ARG);
	  if (!s) risk_exit
	    ("Mistake in sentence %s, in file %s\n",buffer,mapfile,LAST_ARG);
	  strcpy(sentencemapping[t]=malloc(strlen(s)+1),s);
	}
    }
  while(t >= 0);
  if (strncmp(buffer,"THE END",7)) risk_exit
    ("Unexpected eof in %s, should end with \"THE END\"\n",mapfile,LAST_ARG);

  for(t=0;t<NUM_COUNTRIES;t++)
    {
      countries[t].border    = get_picture(countries[t].name,BORDERX,BORDERY);
      if (!strcmp(UNIT0,countries[t].unit)) 
	{
	  countries[t].indexunit = 0;
	  countries[t].value     = V_UNIT0;
	}
      else if (!strcmp(UNIT1,countries[t].unit)) 
	{
	  countries[t].indexunit = 1;
	  countries[t].value     = V_UNIT1;
	}
      else if (!strcmp(UNIT2,countries[t].unit)) 
	{
	  countries[t].indexunit = 2;
	  countries[t].value     = V_UNIT2;
	}
      else risk_exit("Wrong name of %s(unit) for the country %s\n",
		     countries[t].unit,countries[t].name,LAST_ARG);
    }
}


/* Initializes som values */
void initialize()
{
  int                t,t1,h;
  char              *s1,*s2,*s;
  struct Country   **templ;
  struct Continent **tempk;

  /* Continent of free choise */
  strcpy(free_choise.name,"#091_a continent of your own choice");
  free_choise.countries = NULL;
  free_choise.weight    = 0;
  
  /* The Joker */
  strcpy(Joker.name,JOKER);
  Joker.occupant    = NULL;
  Joker.number      = 0;
  Joker.neighbours  = NULL;
  Joker.continent   = NULL;
  strcpy(Joker.unit,JOKER);
  Joker.indexunit   = 0;
  Joker.value       = V_JOKER;
  Joker.border      = get_picture(Joker.name,BORDERX,BORDERY);
  Joker.x = Joker.y = Joker.indexl=0;
  unitp[0]           = get_picture(UNIT0,UNITX,UNITY);
  unitp[1]           = get_picture(UNIT1,UNITX,UNITY);
  unitp[2]           = get_picture(UNIT2,UNITX,UNITY);

  /* Parse what we have read about the countries */
  for(t=0;t<NUM_COUNTRIES;t++)
    {
      /* Put in neighbours */
      s1    = (char *)countries[t].neighbours;
      t1    = num_char(s1,',')+1;
      templ = (struct Country **)malloc((t1+1)*sizeof(struct Country *));
      for(t1=0;s1;t1++)
	{
	  s2 = strchr(s1,',');
	  if (s2) *(s2++) = 0;
	  if (!(templ[t1] = find_country_string(s1)))risk_exit
	    ("Wrong spelled country : %s\n",s1,LAST_ARG);
	  s1=s2;
	}
      templ[t1] = NULL;
      free((char *)countries[t].neighbours);
      countries[t].neighbours = templ;
    }
  /* Fix the missions  */
  for(t=0;missions[t];t++)
    {
      h = 0;
      s = missions[t]->sentence;
      strcpy(s,mappet("#090_Conquer"));
      if (missions[t]->conquestscont)
        {
          s1    = (char *)missions[t]->conquestscont;
          t1    = num_char(s1,',')+1;
          tempk = (struct Continent **)malloc((t1+1)*sizeof(continentp));
          for(t1=0;s1;t1++)
            {
              s2 = strchr(s1,',');
              if (s2) *(s2++)=0;
	      if (s2&&strchr(s2,',')) 
		sprintf(&s[strlen(s)]," %s,",mappet(s1));
	      else if (s2) sprintf(&s[strlen(s)]," %s",mappet(s1));
	      else 
		sprintf(&s[strlen(s)]," %s %s",mappet("#092_and"),mappet(s1));
	      if (!strcmp(s1,"#091_a continent of your own choice"))
		tempk[t1]= &free_choise;
              else if (!(tempk[t1]=find_continent_string(s1))) 
		risk_exit("Mistake in mission : %s [%s]\n",
			  missions[t]->sentence,s1,LAST_ARG);
              s1=s2;
	    }
	  tempk[t1]=NULL;
          free((char *)missions[t]->conquestscont);
	  missions[t]->conquestscont=tempk;
          h++;
        }
      if (missions[t]->conquestsc)
	{
	  sprintf(&s[strlen(s)],mappet("#094_ %d countries")
		  ,missions[t]->conquestsc);
	  if (missions[t]->army > 1) sprintf
	    (&s[strlen(s)],mappet("#093_ witdh %d armies in each"),
	     missions[t]->army);
	  h++;
	}
      if (h!=1) risk_exit
	("Mistake in mission : %s\n",missions[t]->sentence,LAST_ARG);
    }
}

/* Initialize xrisk */
/* All countries,neighbours,continents,missions are read and fixed */
void init_risk()
{
  read_files();
  initialize();
}
